/*
  proto/led.c
  Matlib 2010-02-08
*/


#ifndef _LED_C
#define _LED_C 1

#include <unistd.h>
#include <string.h>

#define LED_ERROR_MIN      -2
#define LED_ERROR_WRITE    -2
#define LED_ERROR_READ     -1
#define LED_ERROR_NOERROR  0
#define LED_ERROR_LOCKED   1
#define LED_ERROR_RANGE    2
#define LED_ERROR_FRANGE   3
#define LED_ERROR_DRANGE   4
#define LED_ERROR_MAX      4

char *led_strerror (int e, char *buffer, size_t buffer_size)
{
  static char *ERROR_CODES [] =
  {
    "write error",
    "read error",
    "operation successful",
    "led device is in use by another process",
    "led number out of range",
    "PWM fill ratio out of range",
    "sequence delay too short"
  };
  if ((e < LED_ERROR_MIN) || (e > LED_ERROR_MAX))
    snprintf (buffer, buffer_size, "unknown error code 0x%x", e);
  else
    strncpy (buffer, ERROR_CODES [e - LED_ERROR_MIN], buffer_size);
  return buffer;
}

int led_add (int fd, unsigned int led_no)
{
  uint8_t buffer [5];
  int e, p;
  buffer [0] = 'l';
  buffer [1] = led_no & 0xff;
  buffer [2] = led_no & 0xff00;
  buffer [3] = led_no & 0xff0000;
  buffer [4] = led_no & 0xff000000;
  for (p = 0; p < 5; p += e)
  {
    e = write (fd, buffer, 5 - p);
    if (e <= 0)
      return LED_ERROR_WRITE;
  }
  if (read (fd, buffer, 1) != 1)
    return LED_ERROR_READ;
  return buffer [0];
}

int led_set (int fd, uint8_t fill)
{
  static uint8_t CMD = 'S';
  uint8_t data;
  if (fill > 100)
    return LED_ERROR_FRANGE;
  if (write (fd, &CMD, 1) != 1)
    return LED_ERROR_WRITE;
  if (write (fd, &fill, 1) != 1)
    return LED_ERROR_WRITE;
  return LED_ERROR_NOERROR;
}


int led_sequence (int fd, unsigned int fill, unsigned int delay)
{
  char buffer [4];
  int p, e;
  if (fill > 100)
    return LED_ERROR_FRANGE;
  if (delay < 100)
    return LED_ERROR_DRANGE;
  buffer [0] = 'B';
  buffer [1] = fill;
  buffer [2] = delay & 0xff;
  buffer [3] = delay & 0xff00;
  for (p = 0; p < 4; p += e)
  {
    e = write (fd, buffer, 4 - p);
    if (e <= 0)
      return LED_ERROR_WRITE;
  }
  return LED_ERROR_NOERROR;
}

int led_loop (int fd, uint8_t step)
{
  static uint8_t CMD = 'L';
  if (write (fd, &CMD, 1) != 1)
    return LED_ERROR_WRITE;
  if (write (fd, &step, 1) != 1)
    return LED_ERROR_WRITE;
  return LED_ERROR_NOERROR;
}

int led_play (int fd)
{
  static uint8_t CMD = 'G';
  if (write (fd, &CMD, 1) != 1)
    return LED_ERROR_WRITE;
}


#endif
