/*
  net.c
  Matlib 2010-02-05
*/


/*
  make_socket (<tcp port>)
  Creates a new listening socket on specified port. Exits program if
  socket cannot be created.
  
  RETURN VALUE: socket
*/

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

int make_socket (char *port)
{
  struct sockaddr_in6 a;
  struct servent *se;
  int i;
  char *tail;
  
  bzero (&a, sizeof (struct sockaddr_in6));
  i = strtol (port, &tail, 0);
  if (*tail == 0)
    a.sin6_port = htons (i);
  else
  {
    se = getservbyname (port, NULL);
    if (se == NULL)
      arguments_error_freeform ("Invalid service name: \"%s\".", port);
    a.sin6_port = se -> s_port;
  }
    
  a.sin6_family = AF_INET6;
  a.sin6_addr = in6addr_any;
  i = socket (PF_INET6, SOCK_STREAM, 0);
  CHECK_ERRORS (i == -1, "socket");
  CHECK_ERROR (bind (i, (struct sockaddr *) (&a), 
     sizeof (struct sockaddr_in6)) != 0, "bind (port %d)", ntohs (a.sin6_port));
  CHECK_ERROR (listen (i, 5), "listen (port %d)", ntohs (a.sin6_port));
  return i;
}



/*
  read_data (<file descriptor>, <buffer>, <bytes to read>)
  Reads a number of bytes of data from file descriptor. On error fd is set
  to -1. When fd == -1 it silently quits filling buffer with 0s.
*/

void read_data (int *fd, void *buffer, size_t size)
{
  size_t p;
  int e;
  if (*fd == -1)
  {
    bzero (buffer, size);
    return;
  }
  p = 0;
loop:
  e = read (*fd, buffer, size - p);
  if (e > 0)
  {
    p += e;
    if (p == size)
      return;
    goto loop;
  }
  close (*fd);
  *fd = -1;
}

