/*
  led-rs232.c
  Matlib 2010-02-04
*/

#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <termios.h>

#include "led.h"



void rs232_led_set (struct DevData *dd)
{
  CHECK_ERROR (ioctl (dd -> fd, TIOCMBIS, 
     &(dd -> arg)) != 0, "%s: led_set: ioctl", dd -> name);
}

void rs232_led_clr (struct DevData *dd)
{
  CHECK_ERROR (ioctl (dd -> fd, TIOCMBIC, 
     &(dd -> arg)) != 0, "%s: led_set: ioctl", dd -> name);
}


void rs232_init (char *name)
{
  struct termios attr;
  int flags, fd;
  struct DevData *dd;
  
  fd = open (name, O_RDWR | O_NOCTTY);
  CHECK_ERROR (fd == -1, "%s: open", name);
  flags = TIOCM_DTR | TIOCM_RTS;
  CHECK_ERROR (ioctl (fd, TIOCMBIC, &flags) != 0, 
     "%s: ioctl: TIOCMBIC", name);
  flags = fcntl (fd, F_GETFL, 0);
  CHECK_ERROR (flags == -1, "%s: fcntl(F_GETFL)", name);
  flags |= O_NONBLOCK;
  CHECK_ERROR (fcntl (fd, F_SETFL, flags) == -1, "%s: fcntl(F_SETFL)", name);
  CHECK_ERROR (ioctl (fd, TIOCEXCL) != 0, "%s: ioctl: TIOCEXCL", name);
  CHECK_ERROR (tcgetattr (fd, &attr) != 0, "%s: tcgetattr", name);
  attr.c_iflag = IGNBRK | IGNPAR;
  attr.c_oflag = NL0 | CR0 | TAB0 | BS0 | VT0 | FF0;
  attr.c_cflag = CS8 | CREAD | HUPCL | CLOCAL;
  attr.c_lflag = 0;
  cfsetispeed (&attr, B115200);
  cfsetospeed (&attr, B115200);
  CHECK_ERROR (tcsetattr (fd, TCSANOW, &attr) != 0, "%s: tcsetattr", name);
  
  dd = devdata_new (name, rs232_led_set, rs232_led_clr);
  dd -> fd = fd;
  dd -> arg = TIOCM_DTR;
  dd = devdata_new (name, rs232_led_set, rs232_led_clr);
  dd -> fd = fd;
  dd -> arg = TIOCM_RTS;
}
